#!/bin/sh
update_config_files ()
{
  package="$1"
  config_file="$2"
  old_path="$3"
  new_path="$4"

  if [ -f "${config_file}" ] && /bin/grep -q "${old_path}" "${config_file}" > /dev/null 2>&1; then
    DATE=$(date +"%Y%m%d%H%M")
    #create backup if there wasn't one created this minute
    if [ ! -f "${config_file}_rpm_update-${DATE}" ]
    then
      cp "${config_file}" "${config_file}_rpm_update-${DATE}" ||
      { logger -t icinga/rpm "Could not create backup file
      ${config_file}_rpm_update-${DATE}" >&2; exit 1; }
    fi

    sed -i "s|${old_path}|${new_path}|g" "${config_file}" \
        || { logger -t icinga/rpm "Could not adapt ${config_file} - please do manually" >&2 ; exit 1; }
    logger -t icinga/rpm "WARNING: ${config_file} adapted to new configuration"
    logger -t icinga/rpm "         (see >rpm -q ${package} --changelog | less<)"
    logger -t icinga/rpm "         Backup file is: ${config_file}_rpm_update-$DATE"
  fi

  if [ -e "${old_path}" ]
  then
    logger -t icinga/rpm "WARNING: ${old_path} has been moved"
    logger -t icinga/rpm "         to ${new_path}"
    logger -t icinga/rpm "         Please check/adapt your configuration/scripts and move the still needed files to the new directory!"
  fi
}

move_file_to_new_location ()
{
  old_path="$1"
  new_path="$2"

  if [ -f "${old_path}" ]
  then
      cp -ua "${old_path}" "${new_path}" || \
      { logger -t icinga/rpm "Could not copy ${old_path} to ${new_path} \
      because a newer version already exists" >&2; exit 1; }
      rm "${old_path}"
  fi
}

# update icinga config files
if [ "$1" = "icinga" ]
then
  # determine if service was running to be able to restart it after the update
  [ -f /run/icinga/icinga.pid ] && service_was_running=true

  [ $service_was_running ] && rcicinga stop >/dev/null 2>&1 ||:

  #############################################
  # update paths to new  run directory
  update_config_files "icinga" "/etc/icinga/icinga.cfg" "/var/spool/icinga/icinga.cmd" "/run/icinga/icinga.cmd"
  # update paths to new  run directory
  #############################################

  #############################################
  # update paths to new  eventhandler directory
  update_config_files "icinga" "/etc/icinga/resource.cfg" "/usr/lib/nagios/plugins/eventhandlers" "/usr/lib/icinga/eventhandler"
  # update paths to new  eventhandler directory
  #############################################

  #############################################
  # update paths to new  spool directory
  update_config_files "icinga" "/etc/icinga/icinga.cfg" "/var/lib/icinga" "/var/spool/icinga"
  update_config_files "icinga" "/etc/icinga/objects/commands.cfg" "/var/lib/icinga" "/var/spool/icinga"
  # update paths to new  spool directory
  #############################################

  #############################################
  # move necessary data to new spool directory
  move_file_to_new_location "/var/lib/icinga/retention.dat" "/var/spool/icinga/"
  move_file_to_new_location "/var/lib/icinga/idomod.tmp" "/var/spool/icinga/"
  move_file_to_new_location "/var/lib/icinga/objects.precache" "/var/spool/icinga/"
  # move necessary data to new  spool directory
  #############################################

  #############################################
  # cleanup remaining unneeded files
  test -e "/var/spool/icinga/icinga.cmd" && rm "/var/spool/icinga/icinga.cmd"
  test -f "/var/lib/icinga/status.dat" && rm "/var/lib/icinga/status.dat"
  test -f "/var/lib/icinga/objects.cache" && rm "/var/lib/icinga/objects.cache"
  # cleanup remaining unneeded files
  #############################################

  # start service if it was running before
  [ $service_was_running ] && rcicinga start >/dev/null 2>&1 ||:
fi

# update icinga-idoutils config files
if [ "$1" = "icinga-idoutils" ]
then
  # determine if service was running
  [ -f /run/icinga/ido2db.pid ] && service_was_running=true

  [ $service_was_running ] && rcido2db stop >/dev/null 2>&1 ||:

  #############################################
  # update paths to new run directory
  update_config_files "icinga-idoutils" "/etc/icinga/ido2db.cfg" "/var/spool/icinga/ido2db.sock" "/run/icinga/ido2db.sock"
  update_config_files "icinga-idoutils" "/etc/icinga/idomod.cfg" "/var/spool/icinga/ido2db.sock" "/run/icinga/ido2db.sock"

  # cleanup old socket file
  test -e "/var/spool/icinga/ido2db.sock" && rm "/var/spool/icinga/ido2db.sock"
  # update paths to new  spool directory
  #############################################

  # start service if it was running before and there wasn't a major upgrade
  [ $service_was_running ] && rcido2db start >/dev/null 2>&1 ||:
fi
